#!/usr/bin/perl -w

# Copyright 2013, 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use List::Util 'max','min','sum';
use Math::Libm 'hypot';
use Math::BaseCnv 'cnv';
use Math::Trig 'rad2deg';
use Math::PlanePath::QuintetCurve 117; # v.117 for level_to_n_range()
use Memoize 'memoize';
use Math::PlanePath::Base::Digits 'bit_split_lowtohigh',
  'digit_split_lowtohigh', 'digit_join_lowtohigh';
use Math::PlanePath;
*_divrem_mutate = \&Math::PlanePath::_divrem_mutate;

use FindBin;
use lib::abs "$FindBin::Bin/../tools";
use MyR5Dragon ':all';
use MyFLAT ':all';

# uncomment this to run the ### lines
use Smart::Comments;

$|=1;


{
  # quartet dir mod 4 state machine
  # 8 states being forward/reverse and dir mod 4

  my @state_table;
  foreach my $dir (0 .. 3) {
    $state_table[$dir]->[0] = ($dir + 0) % 4;
    $state_table[$dir]->[1] = ($dir - 1) % 4 + 4;
    $state_table[$dir]->[2] = ($dir + 0) % 4;
    $state_table[$dir]->[3] = ($dir + 1) % 4;
    $state_table[$dir]->[4] = ($dir + 1) % 4 + 4;

    $state_table[$dir+4]->[0] = ($dir + 1) % 4;
    $state_table[$dir+4]->[1] = ($dir + 1) % 4 + 4;
    $state_table[$dir+4]->[2] = ($dir + 0) % 4 + 4;
    $state_table[$dir+4]->[3] = ($dir - 1) % 4;
    $state_table[$dir+4]->[4] = ($dir + 0) % 4 + 4;
  }

  require Math::NumSeq::PlanePathDelta;
  my $path = Math::PlanePath::QuintetCurve->new;
  my $seq = Math::NumSeq::PlanePathDelta->new(planepath => 'QuintetCurve',
                                              delta_type => 'Dir4');
  print $seq->description,"\n";

  my $LtoH = 0;
  foreach my $target_dir (0 .. 3) {
    my $f = MyFLAT::aref_to_FLAT_DFA
      (\@state_table,
       accepting_list => [$target_dir, $target_dir+4]);
    if ($LtoH) {
      $f = $f->MyFLAT::reverse;
    }
    # $f = $f->MyFLAT::minimize;
    MyFLAT::view($f);

    foreach my $k (0 .. 5) {
      print "k=$k\n";
      my ($n_lo, $n_hi) = $path->level_to_n_range($k);
      foreach my $n ($n_lo .. $n_hi-1) {
        my ($x,$y) = $path->n_to_xy($n);
        my $n5 = sprintf '%0*s', $k, cnv($n,10,5);
        if ($k==0) { $n5 = ''; }
        if ($LtoH) { $n5 = reverse $n5; }

        my $path_dir = $seq->ith($n);
        my $by_path = $seq->ith($n) == $target_dir ?1:0;
        my $by_flat = $f->contains($n5) ?1:0;
        if ($by_path != $by_flat) {
          print "n=$n [$n5] bad flat=$by_flat",
            " path=$by_path dir $path_dir\n";
        }
      }
    }
  }
  exit 0;
}
