#!/usr/bin/perl -w

# Copyright 2013, 2014, 2015, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use warnings;
use List::Util 'max','min';
use Math::Libm 'hypot';
use Math::PlanePath::R5DragonCurve;
use Memoize 'memoize';
use Math::PlanePath::Base::Digits
  'bit_split_lowtohigh', 'digit_join_lowtohigh', 'digit_split_lowtohigh';

use FindBin;
use lib::abs "$FindBin::Bin/../tools";
use MyR5Dragon ':all';

# uncomment this to run the ### lines
# use Smart::Comments;

{
  my $pattern;
  $pattern = 'DDUDDUDU';
  # $pattern = 'DUDUUDUU';
  my $func = sub {
    my ($n) = @_;
    {
      my @digits = digit_split_lowtohigh($n);
      my $rev = 0;
      foreach my $digit (reverse @digits) { # high to low
        if ($rev) { $digit = 8-$digit; }
        my $d = substr($pattern, $digit-1, 1);
        $rev ^= ($digit & 1);
        return ($d eq 'D' ? 90 : -90);
      }
    }
    {
      while ($n) {
        my $digit = $n % 9;
        if ($digit) {
          my $d = substr($pattern, $digit-1, 1);
          return ($d eq 'D' ? 90 : -90);
        }
        $n /= 9;
      }
      return 90;
    }
  };


  my $str = $pattern;
  $str = folding_product($str, $pattern);
  $str = folding_product($str, $pattern);
  $func = sub {
    my ($n) = @_;
    my $d = substr($str, $n-1, 1);
    return ($d eq 'D' ? 90 : -90);
  };

  require Language::Logo;
  my $lo = Logo->new(update => 2, port => 8200 + (time % 100));
  $lo->command("seth 90");
  $lo->command("pendown; hideturtle");
  my $angle = 0;
  foreach my $n (1 .. 9**3) {
    my $turn = $func->($n) // last;
    ### $turn
    $lo->command("forward 10; left $turn");
  }
  $lo->disconnect("Finished...");
  exit 0;

  sub folding_product {
    my ($s, $t) = @_;
    my $ret = $s;
    my $reverse = reverse $s;
    $reverse =~ tr/UD/DU/;
    my $odd = 1;
    foreach my $c (split //, $t) {
      if ($odd) {
        $ret .= $c;
        $ret .= $reverse;
      } else {
        $ret .= $c;
        $ret .= $s;
      }
      $odd ^= 1;
    }
    return $ret;
  }

}
