\\ Copyright 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

read("r5dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------
z_mod_2(z) = (real(z)+imag(z))%2;

z_mod_b_list = [    0    , 1, 1+  I,     I,   2*I, \
                1+2*I, 2+2*I, 2+3*I, 1+3*I, 1+4*I ];
z_mod_b(z) =
{
  for(i=1,5,
     my(r=z_mod_b_list[i]);
     if(z_is_integer((z-r)/b),return(i-1)));
  error("oops, z_mod_b() not found");
}

\\ point  z == b^0 * digitpoint(a0) * I^dir(ak-1..a1) mod b
\\ dir    I^dir(ak-1..a1a0) = s * I^zmod2
\\ dir    I^dir(ak-1..a1) * I^dir(a0) = s * I^zmod2
\\ dir    I^dir(ak-1..a1) = s * I^zmod2 / I^dir(a0) 
\\ z == digitpoint(a0) * s * I^zmod2 / I^dir(a0)  mod b
\\ z*s == digitpoint(a0) / I^dir(a0) * I^zmod2 mod b

check_exact(vector(5,a,a--; digitpoint(a)/I^dir(a)*I^z_mod_2(digitpoint(a))), \
            [0, 1, -1-I, I, 2*I]);
\\      2*I
\\       I
\\       0   1
\\ -1-I
apply(z->(real(z)+2*imag(z))%10,[0, 1, -1-I, I, 2*I])
quit

z_to_low_digit(z) = [0, 3, 2, 1, 4, 0, 3, 2, 1, 4][(real(z)+2*imag(z))%10+1];

check_exact(apply(z_to_low_digit,[0, 1, -1-I, I, 2*I]), \
            [0,1,2,3,4])

r5dragon_unpoint(z,s) = {
  my(debug=0);
  z_is_integer(z)
    || error("r5dragon_unpoint() should be integer z, got ",z);
  s==1 || s==-1
    || error("r5dragon_unpoint() should be s=+/-1, got ",s);
  if(z==0,return([0,0]));
  my(v=List([]));  \\ digits low to high
  if(debug,print("r5dragon_unpoint() z="z"  s="s));
  while(#v<6 && ! (z==0 || (z==I&&d==-I) || (z==-I&&d==I)),
    my(old_z=z,old_s=s);
    my(a=z_to_low_digit(z*s));
    d /= I^dir(a);
    my(delta=d*digitpoint(a));
    z -= delta;
    if(debug,print("at z="old_z" d="old_d
                   "  digit="a" new d="d" delta="delta" to "z));
    z = z/b;
    if(! z_is_integer(z), error("oops, z=",z));
    listput(v,a%5);
    );
  if(debug,print("end z="z" d="d" digits high to low "Vecrev(v)));
  v=Vecrev(v);
  if(z==0 && d==1,   [0, fromdigits(v,5)],
     z==0 && d==-1,  [2, fromdigits(v,5)],
     z==I && d==-I,  [1, 5^#v-fromdigits(v,5)],
     z==-I && d==I,  [3, 5^#v-fromdigits(v,5)],
     error("oops, r5dragon_unpoint() end z=",z," d=",d));
}

{
  for(n=1,20,
     print(r5dragon_unpoint(r5dragon_point(n),1)," ",
           r5dragon_unpoint(r5dragon_point(n),-1)));
}
quit

print()
r5dragon_unpoint(1,1)
r5dragon_unpoint(1,-1)
r5dragon_unpoint(-1,1)
r5dragon_unpoint(-1,-1)

print()
r5dragon_unpoint(1+I,1)
r5dragon_unpoint(1+I,-1)
r5dragon_unpoint(-1-I,1)
r5dragon_unpoint(-1-I,-1)

print()
{
  for(n=1,100,
     my(z=r5dragon_point(n));
     for(arm=0,3,
        my(z=z*I^arm);
        my(s=if(arm==0||arm==2,1,-1));
        \\ if(n%2,,s=-s);
        my(an=r5dragon_unpoint(z,s));
        if(!(an[1]==arm && an[2]==n),
           print("wrong n="n" arm="arm" z="z" s="s"  got "an);
           quit)));
}
