#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::BaseCnv 'cnv';
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon 'xy_rotate_plus90','xy_rotate_minus90','rect_corners_sort';
use MyPrintwrap ':all';


my $want_num_values = 100;
print "% generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::R5DragonCurve->new;

# return the number of sides of the right of segment $x,$y to $dx,$dy
sub xydxdy_RQsides {
  my ($x,$y, $dx,$dy) = @_;
  my $count = 0;
  foreach (1 .. 4) {
    $count += defined($path->xyxy_to_n_either($x,$y, $x+$dx,$y+$dy));
    ($x,$y) = ($x+$dx,$y+$dy);
    ($dx,$dy) = xy_rotate_minus90($dx,$dy); # -90 degrees, to the right
  }
  return $count;
}

# return the number of sides of the left of segment $x,$y to $dx,$dy
sub xydxdy_LQsides {
  my ($x,$y, $dx,$dy) = @_;
  my $count = 0;
  foreach (1 .. 4) {
    $count += defined($path->xyxy_to_n_either($x,$y, $x+$dx,$y+$dy));
    ($x,$y) = ($x+$dx,$y+$dy);
    ($dx,$dy) = xy_rotate_plus90($dx,$dy); # +90 degrees, to the left
  }
  return $count;
}

{
  my @RQsides;
  my %seen;
  for (my $n = 0; @RQsides < $want_num_values; $n++) {
    next unless $path->_UNDOCUMENTED__n_segment_is_right_boundary($n);

    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);

    my $x1 = $x;
    my $y1 = $y;
    my $x2 = $x1 + $dx + $dy;  # opposite corner
    my $y2 = $y1 + $dy - $dx;
    ($x1,$y1, $x2,$y2) = rect_corners_sort($x1,$y1, $x2,$y2);
    next if $seen{"$x1,$y1"}++;

    my $RQsides = xydxdy_RQsides($x,$y, $dx,$dy);
    unless ($RQsides == 1 || $RQsides == 3) {
      die "Oops, got RQsides = $RQsides";
    }
    push @RQsides, $RQsides;
  }

  printwrap_indent("% GP-DEFINE  ");
  printwrap("gen_RQsides_samples = {[");
  $Printwrap = 2;
  foreach my $i (0 .. $#RQsides) {
    printwrap("$RQsides[$i]" . ($i==$#RQsides ? '' : ','));
  }
  $Printwrap = 0;
  printwrap("]};\n");
}
{
  my @LQsides;
  my %seen;
  for (my $n = 0; @LQsides < $want_num_values; $n++) {
    next unless $path->_UNDOCUMENTED__n_segment_is_left_boundary($n);

    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);

    my $x1 = $x;
    my $y1 = $y;
    my $x2 = $x1 + $dx - $dy;  # opposite corner
    my $y2 = $y1 + $dy + $dx;
    ($x1,$y1, $x2,$y2) = rect_corners_sort($x1,$y1, $x2,$y2);
    next if $seen{"$x1,$y1"}++;

    my $LQsides = xydxdy_LQsides($x,$y, $dx,$dy);
    unless ($LQsides == 1 || $LQsides == 3) {
      die "Oops, got LQsides = $LQsides";
    }
    push @LQsides, $LQsides;
  }

  printwrap_indent("% GP-DEFINE  ");
  printwrap("gen_LQsidesInf_samples = {[");
  $Printwrap = 2;
  foreach my $i (0 .. $#LQsides) {
    printwrap("$LQsides[$i]" . ($i==$#LQsides ? '' : ','));
  }
  $Printwrap = 0;
  printwrap("]};\n");
}

print "% end generated\n";
exit 0;
