#!/usr/bin/perl -w

# Copyright 2014, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";
my $chamfer = .15;

my $path = Math::PlanePath::R5DragonCurve->new;
foreach my $level (1,2) {
  my $letter = substr('ZABCDEFG',$level,1);
  my ($n_lo, $n_mid) = $path->level_to_n_range($level);
  my ($xm, $ym) = $path->n_to_xy($n_mid);
  my $n_hi = 2*$n_mid;

  my $xy_is_join = sub {
    my ($x,$y) = @_;
    my @n_list = $path->xy_to_n_list($x,$y);
    return (@n_list > 1
            && $n_list[0] < $n_mid
            && $n_list[1] > $n_mid);
  };

  print "  \\begin{scope}[shift=(pos-$level),rotate={-atan2($ym,$xm)},scale=\\MyScale$letter,\n";
  print "                every circle/.style={radius=\\MyCircleRadius/\\MyScale$letter}]\n";
  print "    % $n_lo to $n_mid to $n_hi\n";

  $Printwrap = 4;
  foreach my $n ($n_lo, $n_mid, $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    printwrap(" \\fill ($x,$y) circle;");
  }
  printwrap_newline();

  printwrap(" \\draw ");
  $Printwrap += 2;
  my $join = '';
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my @n_list = $path->xy_to_n_list($x,$y);
    if ($xy_is_join->($x,$y)) {
      my ($dx,$dy) = $path->n_to_dxdy($n-1);
      my $sx = $x - $chamfer*$dx;
      my $sy = $y - $chamfer*$dy;
      ($dx,$dy) = $path->n_to_dxdy($n);
      my $tx = $x + $chamfer*$dx;
      my $ty = $y + $chamfer*$dy;
      printwrap("$join($sx,$sy)--($tx,$ty)");
    } else {
      printwrap("$join($x,$y)");
    }
    $join = '--';
  }
  printwrap(";\n");

  print "  \\end{scope}\n";
}
print "  % end generated\n";
exit 0;
