#!/usr/bin/perl -w

# Copyright 2017, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::QuintetCurve 117; # v.117 for level_to_n_range()

use lib $FindBin::Bin;
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 3;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $level\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::QuintetCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

$Printwrap = 2;
print "  \\node (start) at (0,0) [my dot];\n";
print "  \\node (end) at ($x_hi,$y_hi) [my dot];\n";

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi - 1) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n + 1);
  $segments->add($x1,$y1, $x2,$y2);
  $x1 = $x_hi - $x1;
  $y1 = $y_hi - $y1;
  $x2 = $x_hi - $x2;
  $y2 = $y_hi - $y2;
  $segments->add($x1,$y1, $x2,$y2);
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";");
my $count = $segments->lines_joined_count_points;
printwrap(" % $count points\n");

print "  % end generated\n";
exit 0;
