#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::QuintetCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon 'chamfer';
use MyPrintwrap ':all';


my $max_level = 4;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::QuintetCurve->new;

foreach my $level (0 .. $max_level) {
  $Printwrap = 2;
  printwrap("\\begin{scope}[shift=(pos-$level)]\n");
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

  $Printwrap = 4;
  printwrap("\\node (start-$level) at (0,0) [my dot];\n");
  printwrap("\\node (end-$level) at ($x_hi,$y_hi) [my dot];\n");
  printwrap("\\draw[my line] ");
  my $join = '';

  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my $semi = ($n == $n_hi ? ';' : '');

    if ($n == $n_lo) {
      printwrap("$join(start-$level)");
    } elsif ($n == $n_hi) {
      printwrap("$join(end-$level);");
    } else {
      printwrap("--($x,$y)");
    }
    $join = '--';
  }
  printwrap_newline();

  print "  \\end{scope}\n";
}
print "  % end generated\n";
exit 0;
